import fconcrete as fc
import json

class API:
    """API is responsable to convert plain text to useful data."""
    def __init__(self, income_text):
        """Reiceive plain text and set attributes with useful data"""
        self.income_text = income_text
        self.status = "OK"
        try:
            concrete_beam_data = json.loads(income_text)["ConcreteBeam"]
            
            to_function = {
                "available_concrete": fc.AvailableConcrete,
                "available_long_steel_bars": fc.AvailableLongConcreteSteelBar,
                "available_transv_steel_bars": fc.AvailableTransvConcreteSteelBar
            }

            to_list = ["available_long_steel_bars_diameters", "available_transv_steel_bars_diameters"]

            for arg, func in to_function.items():
                available_concrete_dict = { k[len(arg)+1:]:([v] if (k in to_list and type(v) != list) else v) for k,v in concrete_beam_data.items() if arg in k }
                concrete_beam_data = {**concrete_beam_data, **{arg: func(**available_concrete_dict)}}

            if concrete_beam_data:
                nodes = [self.getNode(node) for node in concrete_beam_data["nodes"]]
                loads = [self.getLoad(load) for load in concrete_beam_data["loads"]]
                width, height = concrete_beam_data["section"]["width"], concrete_beam_data["section"]["height"]
                other_parameters = { k:v for (k, v) in concrete_beam_data.items() if not k in ["nodes", "loads", "section"]}
                
                concrete_beam = fc.ConcreteBeam(
                        loads = loads,
                        nodes = nodes,
                        section = fc.Rectangle(width,height),
                        **other_parameters
                )
            self.concrete_beam = concrete_beam
        except Exception as excep:
            error = str(excep)
            self.status = error
            
    @staticmethod
    def getNode(noad_dict):
        if noad_dict["type"] == "Free" :
            return getattr(fc.Node, noad_dict["type"])(x=noad_dict["x"])
        return getattr(fc.Node, noad_dict["type"])(x=noad_dict["x"], length=noad_dict["length"])
        
    @staticmethod
    def getLoad(load_dict):
            if load_dict["type"] == "UniformDistributedLoad":
                return fc.Load.UniformDistributedLoad(x_begin=load_dict["x_begin"], x_end=load_dict["x_end"], q=load_dict["q"])
            if load_dict["type"] == "PontualLoad":
                return fc.Load.PontualLoad(x=load_dict["x"], load=load_dict["load"])
            
    def __repr__(self):
        return self.income_text



    


from pytest import approx
import fconcrete as fc

def test_api_concrete_beam():
    income = '''
    {
        "ConcreteBeam": {
            "section": {
                "width": 30,
                "height": 80,
                "type": "Rectangle"
            },
            "nodes": [
                {
                    "x": 0,
                    "length": 20,
                    "type": "SimpleSupport"
                },
                {
                    "x": 400,
                    "length": 40,
                    "type": "SimpleSupport"
                }
            ],
            "loads": [
                {
                    "q": -0.3,
                    "x_begin": 0,
                    "x_end": 400,
                    "type": "UniformDistributedLoad"
                },
                {
                    "load": -0.5,
                    "x": 150,
                    "type": "PontualLoad"
                }
            ],
            "design_factor": 1.4
        }
    }
    '''

    parsed = fc.API(income)
    parsed_concrete_beam = parsed.concrete_beam

    n1 = fc.Node.SimpleSupport(x=0, length=20)
    n2 = fc.Node.SimpleSupport(x=400, length=40)
    f1 = fc.Load.UniformDistributedLoad(-0.3, x_begin=0, x_end=400)
    f2 = fc.Load.PontualLoad(-0.5, x=150)

    concrete_beam2 = fc.ConcreteBeam(
            loads = [f1, f2],
            nodes = [n1, n2],
            section = fc.Rectangle(30,80),
    )

    concrete_beam2.cost

    assert parsed_concrete_beam.cost == concrete_beam2.cost


import numpy as np
from fconcrete import config as c
import matplotlib.pyplot as plt
import time
import ezdxf
import pandas as pd

_Q = c._Q

def cond(x, singular=False, order=0):
    """
    If It is singular, return 1 if x>0 else 0.
    If It is not singular, return x**order if x>0 else 0
    """
    if singular:
        return 1 if x>0 else 0
    return x**order if x>0 else 0

def integrate(f, a, b, N=100):
    """
    Integrate f from a to b in N steps
    """
    x = np.linspace(a, b, N)
    y = np.apply_along_axis(f, 0, np.array([x]))
    return np.trapz(y, dx=(b-a)/(N-1))

def duplicated(array):
    """
    Check if it is duplicated.
    """
    s = np.sort(array, axis=None)
    duplicated = s[:-1][s[1:] == s[:-1]]
    return np.isin(s, duplicated)

def to_unit(input, expected_unit, return_unit=False):
    """
        Convert between unities according to expected_unit and return_unit.

            Call signatures:

                fc.helpers.to_unit(input, expected_unit, return_unit=False)

            >>> unit1 = fc.helpers.to_unit("10cm", "m")
            >>> unit1
            0.1
            
            >>> unit2 = fc.helpers.to_unit(20, "m", return_unit="cm")
            >>> unit2
            2000.0
            
        Parameters
        ----------
        input : number or str
            Represents the input unit of the user.
        
        expected_unit : str
            The expected unit to be given. Useful when input is a number.
            
        return_unit : `bool`, optional
            The desired unit to return

    """
    try:
        input = float(input)
        value = _Q(input, expected_unit)
    except:
        pass
        try:
            value = _Q(input).to(expected_unit)
        except: raise Exception("String does not have valid format. See documentation.")
            
    if return_unit:
        return value.to(return_unit).magnitude
    return value.magnitude
        
def getAxis(xy0=(0,0), xy1=(0,0)):
    """
    Create axis with equal aspect. xy0 and xy1 represent the visible area.
    """
    x0, y0 = xy0
    x1, y1 = xy1
    fig, ax = plt.subplots()
    ax.set_aspect("equal")
    ax.plot([x0, x1], [y0, y1], color="None")
    return fig, ax

def timeit(do=True, name=""):
    """
    Decorator to print the time that the function has taken to execute.
    """
    def inner0(function):
        if not do: return function
        def inner(*args, **kw):
            start = time.time()
            val = function(*args, **kw)
            end = time.time()
            print("{} executed in {}s".format(function.__name__ if name == "" else name, end-start))
            return val
        return inner
    return inner0

# https://gist.github.com/snakers4/91fa21b9dda9d055a02ecd23f24fbc3d
def printProgressBar (iteration, total, prefix = '', suffix = '', decimals = 1, length = 100, fill = '█', printEnd = "\r"):
    """
    Call in a loop to create terminal progress bar
    """
    percent = ("{0:." + str(decimals) + "f}").format(100 * (iteration / float(total)))
    filledLength = int(length * iteration // total)
    bar = fill * filledLength + '-' * (length - filledLength)
    print('\r%s |%s| %s%% %s' % (prefix, bar, percent, suffix), end = printEnd)
    # Print New Line on Complete
    if iteration == total: 
        print()
        
def make_dxf(ax, **options):
    """
        Matplotlib graph to modelspace (preparation to dxf).
        Returns ax and msp.
    """
    msp = options["msp"] if options.get("msp") else False
    scale_y = options["scale_y"] if options.get("scale_y") else 1
    scale_x = options["scale_x"] if options.get("scale_x") else 1
    xy_position = options["xy_position"] if options.get("xy_position") else (0,0)
    
    if msp == False:
        doc = ezdxf.new('AC1032')
        doc.header['$INSUNITS'] = 5
        msp = doc.modelspace()
        
    for element in ax.get_children():
        element_type = str(type(element))
        if element_type == "<class 'matplotlib.lines.Line2D'>":
            xy_data = element.get_xydata()
            xy_data[:, 1] = xy_data[:, 1]*scale_y
            xy_data[:, 0] = xy_data[:, 0]*scale_x
            points = xy_data[np.invert(np.isnan(xy_data[:, 1]))]+xy_position
            msp.add_lwpolyline(points)
        elif element_type == "<class 'matplotlib.patches.Rectangle'>":
            #p1, p2 = element.get_bbox().get_points()
            points = element.get_patch_transform().transform(element.get_path().vertices[:-1]) #np.array([p1, [p1[0], p2[1]], p2, [p2[0], p1[1]], p1])
            points = np.array([*points, points[0]])+xy_position
            msp.add_lwpolyline(points)
            if element.get_hatch():
                hatch = msp.add_hatch()
                hatch.set_pattern_fill('ANSI31', scale=0.5, angle=element.angle)
                hatch.paths.add_polyline_path(points, is_closed=1)
        elif element_type == "<class 'matplotlib.patches.Circle'>":
            msp.add_circle(np.array(element.center)+xy_position, element.radius)
    
    return ax, msp

def to_pandas(array_table):
    df_table = pd.DataFrame(array_table)
    df_table.columns = df_table.iloc[0]
    df_table = df_table.drop(df_table.index[0])
    return df_table



#!/usr/bin/ENV python

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = ['pint==0.11',
                'numpy==1.18.2',
                'matplotlib==3.2.1',
                'pandas==1.0.3',
                'ezdxf==0.11.1'
                ]

setup_requirements = [ ]

test_requirements = [ ]

setup(
    author="Luis Gabriel Gonçalves Coimbra",
    author_email='luiscoimbraeng@outlook.com',
    python_requires='>=3.5',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="Concrete beams according to NBR:6118:2014",
    entry_points={
        'console_scripts': [
            'fconcrete=fconcrete.cli:main',
        ],
    },
    install_requires=requirements,
    license="MIT license",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='fconcrete',
    name='fconcrete',
    packages=find_packages(include=['fconcrete', 'fconcrete.*']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/luisggc/fconcrete',
    version='0.1.1.5',
    zip_safe=False,
    exclude_package_data = {'': ['docs/*', 'tests/*', 'examples/*', 'ENV/*', '.git/*', '.github/*']}
)
